@@ -20,7 +20,10 @@ from urllib.parse import parse_qs, quote
 from azure.core.credentials import AzureSasCredential, AzureNamedKeyCredential, TokenCredential
 from azure.core.exceptions import HttpResponseError
 from azure.core.pipeline import Pipeline
-from azure.core.pipeline.transport import HttpTransport, RequestsTransport  # pylint: disable=non-abstract-transport-import, no-name-in-module
+from azure.core.pipeline.transport import (  # pylint: disable=non-abstract-transport-import, no-name-in-module
+    HttpTransport,
+    RequestsTransport,
+)
 from azure.core.pipeline.policies import (
     AzureSasCredentialPolicy,
     ContentDecodePolicy,
@@ -64,34 +67,51 @@ _SERVICE_PARAMS = {
 }
 
 
-class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-attributes
+class StorageAccountHostsMixin(object):
+
     _client: Any
+    _hosts: Dict[str, str]
+
     def __init__(
         self,
         parsed_url: Any,
         service: str,
-        credential: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "AsyncTokenCredential", TokenCredential]] = None, # pylint: disable=line-too-long
-        **kwargs: Any
+        credential: Optional[
+            Union[
+                str,
+                Dict[str, str],
+                AzureNamedKeyCredential,
+                AzureSasCredential,
+                "AsyncTokenCredential",
+                TokenCredential,
+            ]
+        ] = None,
+        **kwargs: Any,
     ) -> None:
         self._location_mode = kwargs.get("_location_mode", LocationMode.PRIMARY)
-        self._hosts = kwargs.get("_hosts")
+        self._hosts = kwargs.get("_hosts", {})
         self.scheme = parsed_url.scheme
+        self._is_localhost = False
 
         if service not in ["blob", "queue", "file-share", "dfs"]:
             raise ValueError(f"Invalid service: {service}")
-        service_name = service.split('-')[0]
+        service_name = service.split("-")[0]
         account = parsed_url.netloc.split(f".{service_name}.core.")
 
         self.account_name = account[0] if len(account) > 1 else None
-        if not self.account_name and parsed_url.netloc.startswith("localhost") \
-                or parsed_url.netloc.startswith("127.0.0.1"):
+        if (
+            not self.account_name
+            and parsed_url.netloc.startswith("localhost")
+            or parsed_url.netloc.startswith("127.0.0.1")
+        ):
+            self._is_localhost = True
             self.account_name = parsed_url.path.strip("/")
 
         self.credential = _format_shared_key_credential(self.account_name, credential)
         if self.scheme.lower() != "https" and hasattr(self.credential, "get_token"):
             raise ValueError("Token credential is only supported with HTTPS.")
 
-        secondary_hostname = None
+        secondary_hostname = ""
         if hasattr(self.credential, "account_name"):
             self.account_name = self.credential.account_name
             secondary_hostname = f"{self.credential.account_name}-secondary.{service_name}.{SERVICE_HOST_BASE}"
@@ -101,7 +121,7 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
                 secondary_hostname = parsed_url.netloc.replace(account[0], account[0] + "-secondary")
             if kwargs.get("secondary_hostname"):
                 secondary_hostname = kwargs["secondary_hostname"]
-            primary_hostname = (parsed_url.netloc + parsed_url.path).rstrip('/')
+            primary_hostname = (parsed_url.netloc + parsed_url.path).rstrip("/")
             self._hosts = {LocationMode.PRIMARY: primary_hostname, LocationMode.SECONDARY: secondary_hostname}
 
         self._sdk_moniker = f"storage-{service}/{VERSION}"
@@ -114,70 +134,76 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
     def __exit__(self, *args):
         self._client.__exit__(*args)
 
-    def close(self):
-        """ This method is to close the sockets opened by the client.
+    def close(self) -> None:
+        """This method is to close the sockets opened by the client.
         It need not be used when using with a context manager.
         """
         self._client.close()
 
     @property
-    def url(self):
+    def url(self) -> str:
         """The full endpoint URL to this entity, including SAS token if used.
 
         This could be either the primary endpoint,
         or the secondary endpoint depending on the current :func:`location_mode`.
-        :returns: The full endpoint URL to this entity, including SAS token if used.
+
+        :return: The full endpoint URL to this entity, including SAS token if used.
         :rtype: str
         """
-        return self._format_url(self._hosts[self._location_mode])
+        return self._format_url(self._hosts[self._location_mode])   # type: ignore
 
     @property
-    def primary_endpoint(self):
+    def primary_endpoint(self) -> str:
         """The full primary endpoint URL.
 
+        :return: The full primary endpoint URL.
         :rtype: str
         """
-        return self._format_url(self._hosts[LocationMode.PRIMARY])
+        return self._format_url(self._hosts[LocationMode.PRIMARY])  # type: ignore
 
     @property
-    def primary_hostname(self):
+    def primary_hostname(self) -> str:
         """The hostname of the primary endpoint.
 
+        :return: The hostname of the primary endpoint.
         :rtype: str
         """
         return self._hosts[LocationMode.PRIMARY]
 
     @property
-    def secondary_endpoint(self):
+    def secondary_endpoint(self) -> str:
         """The full secondary endpoint URL if configured.
 
         If not available a ValueError will be raised. To explicitly specify a secondary hostname, use the optional
         `secondary_hostname` keyword argument on instantiation.
 
+        :return: The full secondary endpoint URL.
         :rtype: str
-        :raise ValueError:
+        :raise ValueError: If no secondary endpoint is configured.
         """
         if not self._hosts[LocationMode.SECONDARY]:
             raise ValueError("No secondary host configured.")
-        return self._format_url(self._hosts[LocationMode.SECONDARY])
+        return self._format_url(self._hosts[LocationMode.SECONDARY])    # type: ignore
 
     @property
-    def secondary_hostname(self):
+    def secondary_hostname(self) -> Optional[str]:
         """The hostname of the secondary endpoint.
 
         If not available this will be None. To explicitly specify a secondary hostname, use the optional
         `secondary_hostname` keyword argument on instantiation.
 
+        :return: The hostname of the secondary endpoint, or None if not configured.
         :rtype: Optional[str]
         """
         return self._hosts[LocationMode.SECONDARY]
 
     @property
-    def location_mode(self):
+    def location_mode(self) -> str:
         """The location mode that the client is currently using.
 
         By default this will be "primary". Options include "primary" and "secondary".
 
+        :return: The current location mode.
         :rtype: str
         """
 
@@ -200,11 +226,16 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
         return self._client._config.version  # pylint: disable=protected-access
 
     def _format_query_string(
-        self, sas_token: Optional[str],
-        credential: Optional[Union[str, Dict[str, str], "AzureNamedKeyCredential", "AzureSasCredential", TokenCredential]],  # pylint: disable=line-too-long
+        self,
+        sas_token: Optional[str],
+        credential: Optional[
+            Union[str, Dict[str, str], "AzureNamedKeyCredential", "AzureSasCredential", TokenCredential]
+        ],
         snapshot: Optional[str] = None,
-        share_snapshot: Optional[str] = None
-    ) -> Tuple[str, Optional[Union[str, Dict[str, str], "AzureNamedKeyCredential", "AzureSasCredential", TokenCredential]]]:  # pylint: disable=line-too-long
+        share_snapshot: Optional[str] = None,
+    ) -> Tuple[
+        str, Optional[Union[str, Dict[str, str], "AzureNamedKeyCredential", "AzureSasCredential", TokenCredential]]
+    ]:
         query_str = "?"
         if snapshot:
             query_str += f"snapshot={snapshot}&"
@@ -212,7 +243,8 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
             query_str += f"sharesnapshot={share_snapshot}&"
         if sas_token and isinstance(credential, AzureSasCredential):
             raise ValueError(
-                "You cannot use AzureSasCredential when the resource URI also contains a Shared Access Signature.")
+                "You cannot use AzureSasCredential when the resource URI also contains a Shared Access Signature."
+            )
         if _is_credential_sastoken(credential):
             credential = cast(str, credential)
             query_str += credential.lstrip("?")
@@ -222,13 +254,16 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
         return query_str.rstrip("?&"), credential
 
     def _create_pipeline(
-        self, credential: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, TokenCredential]] = None, # pylint: disable=line-too-long
-        **kwargs: Any
+        self,
+        credential: Optional[
+            Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, TokenCredential]
+        ] = None,
+        **kwargs: Any,
     ) -> Tuple[StorageConfiguration, Pipeline]:
         self._credential_policy: Any = None
         if hasattr(credential, "get_token"):
-            if kwargs.get('audience'):
-                audience = str(kwargs.pop('audience')).rstrip('/') + DEFAULT_OAUTH_SCOPE
+            if kwargs.get("audience"):
+                audience = str(kwargs.pop("audience")).rstrip("/") + DEFAULT_OAUTH_SCOPE
             else:
                 audience = STORAGE_OAUTH_SCOPE
             self._credential_policy = StorageBearerTokenCredentialPolicy(cast(TokenCredential, credential), audience)
@@ -262,22 +297,18 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
             config.logging_policy,
             StorageResponseHook(**kwargs),
             DistributedTracingPolicy(**kwargs),
-            HttpLoggingPolicy(**kwargs)
+            HttpLoggingPolicy(**kwargs),
         ]
         if kwargs.get("_additional_pipeline_policies"):
             policies = policies + kwargs.get("_additional_pipeline_policies")  # type: ignore
         config.transport = transport  # type: ignore
         return config, Pipeline(transport, policies=policies)
 
-    def _batch_send(
-        self,
-        *reqs: "HttpRequest",
-        **kwargs: Any
-    ) -> Iterator["HttpResponse"]:
+    def _batch_send(self, *reqs: "HttpRequest", **kwargs: Any) -> Iterator["HttpResponse"]:
         """Given a series of request, do a Storage batch call.
 
         :param HttpRequest reqs: A collection of HttpRequest objects.
-        :returns: An iterator of HttpResponse objects.
+        :return: An iterator of HttpResponse objects.
         :rtype: Iterator[HttpResponse]
         """
         # Pop it here, so requests doesn't feel bad about additional kwarg
@@ -286,35 +317,29 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
 
         request = self._client._client.post(  # pylint: disable=protected-access
             url=(
-                f'{self.scheme}://{self.primary_hostname}/'
+                f"{self.scheme}://{self.primary_hostname}/"
                 f"{kwargs.pop('path', '')}?{kwargs.pop('restype', '')}"
                 f"comp=batch{kwargs.pop('sas', '')}{kwargs.pop('timeout', '')}"
             ),
             headers={
-                'x-ms-version': self.api_version,
-                "Content-Type": "multipart/mixed; boundary=" + _get_batch_request_delimiter(batch_id, False, False)
-            }
+                "x-ms-version": self.api_version,
+                "Content-Type": "multipart/mixed; boundary=" + _get_batch_request_delimiter(batch_id, False, False),
+            },
         )
 
         policies = [StorageHeadersPolicy()]
         if self._credential_policy:
             policies.append(self._credential_policy)
 
-        request.set_multipart_mixed(
-            *reqs,
-            policies=policies,
-            enforce_https=False
-        )
+        request.set_multipart_mixed(*reqs, policies=policies, enforce_https=False)
 
-        Pipeline._prepare_multipart_mixed_request(request) # pylint: disable=protected-access
+        Pipeline._prepare_multipart_mixed_request(request)  # pylint: disable=protected-access
         body = serialize_batch_body(request.multipart_mixed_info[0], batch_id)
         request.set_bytes_body(body)
 
         temp = request.multipart_mixed_info
         request.multipart_mixed_info = None
-        pipeline_response = self._pipeline.run(
-            request, **kwargs
-        )
+        pipeline_response = self._pipeline.run(request, **kwargs)
         response = pipeline_response.http_response
         request.multipart_mixed_info = temp
 
@@ -326,12 +351,11 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
                 parts = list(response.parts())
                 if any(p for p in parts if not 200 <= p.status_code < 300):
                     error = PartialBatchErrorException(
-                        message="There is a partial failure in the batch operation.",
-                        response=response, parts=parts
+                        message="There is a partial failure in the batch operation.", response=response, parts=parts
                     )
                     raise error
                 return iter(parts)
-            return parts
+            return parts  # type: ignore [no-any-return]
         except HttpResponseError as error:
             process_storage_error(error)
 
@@ -341,6 +365,7 @@ class TransportWrapper(HttpTransport):
     by a `get_client` method does not close the outer transport for the parent
     when used in a context manager.
     """
+
     def __init__(self, transport):
         self._transport = transport
 
@@ -356,13 +381,15 @@ class TransportWrapper(HttpTransport):
     def __enter__(self):
         pass
 
-    def __exit__(self, *args):  # pylint: disable=arguments-differ
+    def __exit__(self, *args):
         pass
 
 
 def _format_shared_key_credential(
-    account_name: str,
-    credential: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "AsyncTokenCredential", TokenCredential]] = None, # pylint: disable=line-too-long
+    account_name: Optional[str],
+    credential: Optional[
+        Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "AsyncTokenCredential", TokenCredential]
+    ] = None,
 ) -> Any:
     if isinstance(credential, str):
         if not account_name:
@@ -381,9 +408,13 @@ def _format_shared_key_credential(
 
 def parse_connection_str(
     conn_str: str,
-    credential: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, TokenCredential]], # pylint: disable=line-too-long
-    service: str
-) -> Tuple[str, Optional[str], Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, TokenCredential]]]: # pylint: disable=line-too-long
+    credential: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, TokenCredential]],
+    service: str,
+) -> Tuple[
+    str,
+    Optional[str],
+    Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, TokenCredential]],
+]:
     conn_str = conn_str.rstrip(";")
     conn_settings_list = [s.split("=", 1) for s in conn_str.split(";")]
     if any(len(tup) != 2 for tup in conn_settings_list):
@@ -405,14 +436,11 @@ def parse_connection_str(
         if endpoints["secondary"] in conn_settings:
             raise ValueError("Connection string specifies only secondary endpoint.")
         try:
-            primary =(
+            primary = (
                 f"{conn_settings['DEFAULTENDPOINTSPROTOCOL']}://"
                 f"{conn_settings['ACCOUNTNAME']}.{service}.{conn_settings['ENDPOINTSUFFIX']}"
             )
-            secondary = (
-                f"{conn_settings['ACCOUNTNAME']}-secondary."
-                f"{service}.{conn_settings['ENDPOINTSUFFIX']}"
-            )
+            secondary = f"{conn_settings['ACCOUNTNAME']}-secondary." f"{service}.{conn_settings['ENDPOINTSUFFIX']}"
         except KeyError:
             pass
 
@@ -432,7 +460,7 @@ def parse_connection_str(
 
 
 def create_configuration(**kwargs: Any) -> StorageConfiguration:
-     # Backwards compatibility if someone is not passing sdk_moniker
+    # Backwards compatibility if someone is not passing sdk_moniker
     if not kwargs.get("sdk_moniker"):
         kwargs["sdk_moniker"] = f"storage-{kwargs.pop('storage_sdk')}/{VERSION}"
     config = StorageConfiguration(**kwargs)
